"{ Package: 'stx:goodies/xtreams/transforms/tests' }"

"{ NameSpace: Xtreams }"

Object subclass:#MessagePackMarshalerTest
	instanceVariableNames:'stream marshaling log bytes marshaler'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Transforms-Tests'
!


!MessagePackMarshalerTest methodsFor:'initialize-release'!

setUp
	marshaler := MessagePackMarshaler new.
	stream := ByteArray new writing.
	marshaling := stream marshaling: marshaler.
	bytes := nil.
	log := nil
! !

!MessagePackMarshalerTest methodsFor:'private'!

analyse
	"Run an analysis on the stream contents to ensure it has integrity"
	bytes := stream conclusion.
	log := String new writing.
	(ObjectAnalyseStream on: bytes reading marshaler: marshaler class new) do: [:each | log write: each].
	log := log conclusion.
	marshaling := bytes reading marshaling: marshaler class new
! !

!MessagePackMarshalerTest methodsFor:'tests'!

testArray
	| fixed small big |
	big := Array new: 65537 withAll: 1.
	small := (1 to: 255) asArray.
	fixed := Array with: small with: big.
	marshaling put: fixed.
	marshaling put: small.
	marshaling put: big.
	self analyse.
	self assert: marshaling get = fixed.
	self assert: marshaling get = small.
	self assert: marshaling get = big
!

testDictionary
	| fixed small big |
	big := Dictionary new: 65537.
	1 to: 65537 do: [:i | big at: i put: 1].
	small := Dictionary new: 255.
	1 to: 255 do: [:i | small at: i put: i].
	fixed := Dictionary new
		at: 1 put: big;
		at: 2 put: small;
		yourself.
	marshaling put: fixed.
	marshaling put: small.
	marshaling put: big.
	self analyse.
	self assert: marshaling get = fixed.
	self assert: marshaling get = small.
	self assert: marshaling get = big
!

testFloatDouble
	marshaling write: #(3.14 3.14159 3.141592653589793).
	self analyse.
	self assert: marshaling rest = #(3.14 3.14159 3.141592653589793)
!

testIntegers
	| inrange outofrange |
	outofrange := #(18446744073709551616 -9223372036854775809).
	inrange := #(
		0 1 126 127 128 255 256
		32767 32768 32769
		65535 65536 65537
		4294967294 4294967295 4294967296
		18446744073709551614 18446744073709551615
		-1 -2 -32 -33
		-127 -128 -255 -256
		-32767 -32768 -32769
		-65535 -65536 -65537
		-2147483647 -2147483648 -2147483649
		-9223372036854775807 -9223372036854775808).

	marshaling write: inrange.
	outofrange do: [:each | self should: [marshaling put: each] raise: Error].
	self analyse.
	self assert: marshaling rest = inrange
!

testRaw
	| big |
	big := ByteArray new: 100000 withAll: 1.
	marshaling put: #[10 255 0].
	marshaling put: (ByteArray withAll: (1 to: 255)).
	marshaling put: big.
	self analyse.
	self assert: marshaling get = #[10 255 0].
	self assert: marshaling get = (ByteArray withAll: (1 to: 255)).
	self assert: marshaling get = big
!

testSingletons
	marshaling put: nil.
	marshaling put: true.
	marshaling put: false.
	self analyse.
	self assert: marshaling get = nil.
	self assert: marshaling get = true.
	self assert: marshaling get = false
! !

!MessagePackMarshalerTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

